RandRat := function()
    return Random([0 .. 10000])/10000;
end;

InstallGlobalFunction(CreateSimpleFlipPropose,
    function(prob)
        return function(cnf)
            local rn, point;

            rn := RandRat();
            if rn >= prob and CountObjects(cnf) > 0 then
                point := ChooseRandomObject(cnf);
                return DelObjectChange(point);
            else
                point := RandomNewObject(cnf, RandRat);
                return AddObjectChange(point);
            fi;
        end;
    end);

InstallGlobalFunction(CreateStraussCheck,
    function(beta, gamma, rad)
        return function(cnf, change)
            local t;

            t := CloseObjects(cnf, change!.obj, rad);
            if IsAddObjectChange(change) then
                return Minimum(1, ConfigWindowArea(cnf) * beta * gamma ^ t /
                               CountObjects(cnf));
            else
                return Minimum(1, (CountObjects(cnf) + 1) /
                              (ConfigWindowArea(cnf) * beta * gamma ^ t));
            fi;
        end;
    end);

InstallGlobalFunction(CreateRedBlackPropose,
    function(rad, mean_red, mean_black)
        local red_rnd, black_rnd;

        red_rnd := CreatePoissonRNG(mean_red);
        black_rnd := CreatePoissonRNG(mean_black);
        return function(cnf)
            local del, add, obj, num, i;

            # if isRedIter is not set ...
            if not IsBound(cnf!.isRedIter) then
                cnf!.isRedIter := false;
            fi;
            if cnf!.isRedIter then
                del := GetObjectsWithMark(cnf, "red");
                add := [];
                ## replace points outside of 
                num := red_rnd();
                for i in [ 1 .. num ] do
                    obj := RandomNewObject(cnf, RandRat);
                    AddMark(obj, "red");
                    if CloseObjectsWithMark(cnf, obj, rad, "black") = 0 then
                        Append(add, [ obj ]);
                    fi;
                od;
            else
                del := GetObjectsWithMark(cnf, "black");
                add := [];
                ## replace points outside of 
                num := black_rnd();
                for i in [ 1 .. num ] do
                    obj := RandomNewObject(cnf, RandRat);
                    AddMark(obj, "black");
                    if CloseObjectsWithMark(cnf, obj, rad, "red") = 0 then
                        Append(add, [ obj ]);
                    fi;
                od;
            fi;
            cnf!.isRedIter := not cnf!.isRedIter;
            return ReplaceMultipleObjectsChange(add, del);
        end;
    end);

InstallGlobalFunction(CreateFixedCheck,
    function(prob)
        return function(cnf, change)
            return prob;
        end;
    end);
