CreateUniformRNG := function(start, endp)
  local scale;

  scale := (endp - start)/10000;
  return function()
    return start + scale * Random([0 .. 10000]);
  end;
end;

CreatePoissonRNG := function(mean)
  local e, lastcoeff, cumm, fmean, poiss10, poiss, ten;

  e := FLOAT_INT(27182818)/FLOAT_INT(10000000);
  ten := FLOAT_INT(10);
  if IS_FLOAT(mean) then
    fmean := mean;
  else
    fmean := FLOAT_INT(NumeratorRat(mean))/FLOAT_INT(DenominatorRat(mean));
  fi;
  if fmean <= ten then
    lastcoeff := e^(-fmean);
    cumm := [ lastcoeff ];
    return function()
      local u, k;

      u := FLOAT_INT(Random([0 .. 10000])) / FLOAT_INT(10000);
      k := -1;
      repeat
        k := k + 1;
        # make sure we have the correct coefficient in the list ...
        if Length(cumm) < k + 1 then
          lastcoeff := lastcoeff * fmean / FLOAT_INT(k);
          Append(cumm, [ cumm[k] + lastcoeff]);
        fi;
      until u <= cumm[k+1];
      return k;
    end;
  else
    poiss10 := CreatePoissonRNG(10);
    poiss := CreatePoissonRNG(fmean mod ten);
    return function()
      local count, total;

      count := fmean;
      total := 0;
      while count > ten do
        total := total + poiss10();
        count := count - ten;
      od;
      return total + poiss();
    end;
  fi;
end;
