#### AddObjectChange

AddObjectChangeType := NewType(ChangeFamily, IsAddObjectChange);
InstallGlobalFunction(AddObjectChange,
    function(picobj)
        return Objectify(AddObjectChangeType, rec(obj:=picobj));
    end);

InstallMethod(ApplyChange, "apply an add object change",
    true, [ IsAddObjectChange, IsConfiguration ], 0,
    function (chng, cnf)
        AddObject(cnf, chng!.obj);
    end);
InstallMethod(RevertChange, "apply an add object change",
    true, [ IsAddObjectChange, IsConfiguration ], 0,
    function (chng, cnf)
        DelObject(cnf, chng!.obj);
    end);
InstallMethod(PrintObj, "",
    true, [ IsAddObjectChange ], 0,
    function (chng)
        Print("AddObjectChange( ", chng!.obj, " )");
    end);

#### DelObjectChange

DelObjectChangeType := NewType(ChangeFamily, IsDelObjectChange);
InstallGlobalFunction(DelObjectChange,
    function(picobj)
        return Objectify(DelObjectChangeType, rec(obj:=picobj));
    end);

InstallMethod(ApplyChange, "apply a del object change",
    true, [ IsDelObjectChange, IsConfiguration ], 0,
    function (chng, cnf)
        DelObject(cnf, chng!.obj);
    end);
InstallMethod(RevertChange, "apply a del object change",
    true, [ IsDelObjectChange, IsConfiguration ], 0,
    function (chng, cnf)
        AddObject(cnf, chng!.obj);
    end);

InstallMethod(PrintObj, "",
    true, [ IsDelObjectChange ], 0,
    function (chng)
        Print("DelObjectChange( ", chng!.obj, " )");
    end);

#### ReplaceObjectChange

ReplaceObjectChangeType :=
    NewType(ChangeFamily, IsReplaceObjectChange);
InstallGlobalFunction(ReplaceObjectChange,
    function(addobj, delobj)
        return Objectify(ReplaceObjectChangeType,
			 rec(addobj:=addobj, delobj:=delobj));
    end);

InstallMethod(ApplyChange, "apply a replace object change",
    true, [ IsReplaceObjectChange, IsConfiguration ], 0,
    function (chng, cnf)
        DelObject(cnf, chng!.delobj);
        AddObject(cnf, chng!.addobj);
    end);
InstallMethod(RevertChange, "apply a replace object change",
    true, [ IsReplaceObjectChange, IsConfiguration ], 0,
    function (chng, cnf)
        DelObject(cnf, chng!.addobj);
        AddObject(cnf, chng!.delobj);
    end);
InstallMethod(PrintObj, "",
    true, [ IsReplaceObjectChange ], 0,
    function (chng)
        Print("ReplaceObjectChange( ", chng!.addobj, ", ", chng!.delobj, " )");
    end);


#### ReplaceMultipleObjectsChange


ReplaceMultipleObjectsChangeType :=
    NewType(ChangeFamily, IsReplaceMultipleObjectsChange);
InstallGlobalFunction(ReplaceMultipleObjectsChange,
    function(addobjs, delobjs)
        return Objectify(ReplaceMultipleObjectsChangeType,
			 rec(addobjs:=addobjs, delobjs:=delobjs));
    end);

InstallMethod(ApplyChange, "apply a replace multiple objects change",
    true, [ IsReplaceMultipleObjectsChange, IsConfiguration ], 0,
    function (chng, cnf)
	local obj;

	for obj in chng!.delobjs do
	    DelObject(cnf, obj);
        od;
	for obj in chng!.addobjs do
	    AddObject(cnf, obj);
        od;
    end);
InstallMethod(RevertChange, "apply a replace multiple objects change",
    true, [ IsReplaceMultipleObjectsChange, IsConfiguration ], 0,
    function (chng, cnf)
	local obj;

	for obj in chng!.addobjs do
	    DelObject(cnf, obj);
        od;
	for obj in chng!.delobjs do
	    AddObject(cnf, obj);
        od;
    end);
InstallMethod(PrintObj, "",
    true, [ IsReplaceMultipleObjectsChange ], 0,
    function (chng)
        Print("ReplaceReplaceObjectsChange( ", chng!.addobjs, ", ",
              chng!.delobjs, " )");
    end);

